//****************************************************************************
// Name:        down_data.cpp
// Author :     John Q Metro
// Purpose :    Enumerations and types to help for downloading
// Created:     December 5, 2013
// Updated:     August 11, 2015
//****************************************************************************
#ifndef DOWN_DATA_H_INCLUDED
  #include "down_data.h"
#endif // DOWN_DATA_H_INCLUDED
//----------------------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
//----------------------------------
#include <assert.h>
#include <cstdlib>
//****************************************************************************
// download status to string
wxString jfDown::DownStatus2String(const DownStatus& inval) {
  switch (inval) {
    case NOTSTARTED : return wxT("Download not started.");
    case FETCHING   : return wxT("Downloading now.");
    case DONE_OK    : return wxT("Download successful.");
    case DONE_FAILED: return wxT("Download failed.");
    default : assert(false);
  }
}
//----------------------------------------------------------
// fetch result to string
wxString jfDown::FetchResult2String(const FetchResult& inval) {
  switch (inval) {
    case FETCHED     : return wxT("Page fetched succesfully!");
    case TIMEDOUT    : return wxT("The page request timed out!");
    case REDIRECTED  : return wxT("The page request was redirected!");
    case MISSING     : return wxT("The page is missing!");
    case FAILED      : return wxT("Attempting to get the page just failed!");
    case BLOCKED     : return wxT("Some other fetching error happened!");
    case ABORTED     : return wxT("User asked for download to stop!");
    case PARSE_ERROR : return wxT("Parsing Error (page was not as expected)!");
    case SSL_ERROR   : return wxT("SSL related error!");
    default : assert(false);
  }
}
//===========================================================================
// Retry Constants
//----------------------------
const size_t jfDown::minor_retries = 4;
const size_t jfDown::major_retries = 4;
const size_t jfDown::timeout = 15;
//-----------------------------
size_t jfDown::SleepPause() {
  size_t spause = 500;
  std::srand(time(NULL));
  spause += (std::rand() % 1000);
  return spause;
}
//===========================================================================
// new pauser constructors and methods
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// default constructor
jfNewPauser::jfNewPauser() {
  msg_target = NULL;
  special_stop = false;
  status = jfDown::WORKING;
  which_option = false;
}
//-----------------------------------------
// saves having to set the target
jfNewPauser::jfNewPauser(wxWindow* target_in) {
  msg_target = target_in;
  special_stop = false;
  status = jfDown::WORKING;
  which_option = false;
}
//-----------------------------------------
// just in case we have to set the target anyway
bool jfNewPauser::SetTarget(wxWindow* new_target) {
  wxMutexLocker mlocker(statlock);
  msg_target = new_target;
  return (msg_target!=NULL);
}
//-----------------------------------------
bool jfNewPauser::SetOption(bool value) {
  wxMutexLocker mlocker(statlock);
  which_option = value;
  return value;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++
// external methods called to control the thread
//-----------------------------------------
bool jfNewPauser::Pause() {
  wxMutexLocker mlock(statlock);
  if (status!=jfDown::WORKING) return false;
  status = jfDown::PAUSING;
  pauselock.Lock();
  return true;
}
//-----------------------------------------
bool jfNewPauser::Resume() {
  wxMutexLocker mlock(statlock);
  if ((status!=jfDown::PAUSED) && (status!=jfDown::PAUSING)) return false;
  status = jfDown::WORKING;
  pauselock.Unlock();
  return true;
}
//-----------------------------------------
bool jfNewPauser::Stop() {
  const wxString fname = wxT("jfNewPauser::Stop");
  wxMutexLocker mlock(statlock);
  /**/jfXLogMessage(fname,1);
  if ((status==jfDown::STOPPING) || (status==jfDown::STOPPED)) return false;
  /**/jfXLogMessage(fname,2);
  // pause means resume...
  bool ostatus = ((status==jfDown::PAUSED) || (status==jfDown::PAUSING));
  status = jfDown::STOPPING;
  /**/jfXLogMessageB(fname,3,ostatus);
  if  (ostatus) pauselock.Unlock();
  return true;
}
//-----------------------------------------
// used for stoppoing only when a category is done
bool jfNewPauser::SpecialStop() {
  wxMutexLocker mlock(statlock);
  if ((status==jfDown::STOPPING) || (status==jfDown::STOPPED)) return false;
  if (special_stop) return false;
  return (special_stop = true);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++
// thread calls to pause or get stop info
//-----------------------------------------
bool jfNewPauser::PauseHaltCheck() {
  statlock.Lock();
  // first stop check
  if (status==jfDown::STOPPING) {
    status = jfDown::STOPPED;
    statlock.Unlock();
    return true;
  }
  // pause check
  if (status==jfDown::PAUSING) {
    SendPauseSignal();
    status = jfDown::PAUSED;
    statlock.Unlock();
    // here is how we actually pause
    pauselock.Lock();
    pauselock.Unlock();
    // resume checks
    statlock.Lock();
    // second stop check
    if (status==jfDown::STOPPING) {
      status = jfDown::STOPPED;
      statlock.Unlock();
      return true;
    }
    SendResumeSignal();
  }
  statlock.Unlock();
  return false;
}
//-----------------------------------------
bool jfNewPauser::SpecialStopCheck() {
  wxMutexLocker mlock(statlock);
  if (special_stop) status = jfDown::STOPPED;
  return special_stop;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++
// sending a pause/resume message
//-----------------------------------------
void jfNewPauser::SendPauseSignal() {
  if (msg_target!=NULL) {
    // creating
    wxCommandEvent eventx(jf_DOWNL_PAUSE_EVENT, msg_target->GetId());
    eventx.SetInt((int)which_option);
    // sending the message
    wxPostEvent(msg_target,eventx);
  }
}
//-----------------------------------------
void jfNewPauser::SendResumeSignal() {
  if (msg_target!=NULL) {
    // creating
    wxCommandEvent eventx(jf_DOWNL_RESUME_EVENT, msg_target->GetId());
    eventx.SetInt((int)which_option);
    // sending the message
    wxPostEvent(msg_target,eventx);
  }
}
//===========================================================================
jfDownloadParams::jfDownloadParams() {
  pauselink = NULL;
  msg_target = NULL;
  CheckMissing = NULL;
  CheckTruncated = NULL;
  fetchlimit = 300000;
  pageskip = false;
  skipmissing = false;
  no_major_retry = false;
}
//===========================================================================
// special

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// constructors
//--------------------------------
jfThreadEvent::jfThreadEvent(wxEventType commandType,int id):wxEvent(id,commandType) {
  int1 = int2 = -1;
  option = false;
  ferr = jfDown::FETCHED;
}
//--------------------------------
jfThreadEvent::jfThreadEvent(wxEventType commandType,int id, bool indata):wxEvent(id,commandType) {
  int1 = int2 = -1;
  option = indata;
  ferr = jfDown::FETCHED;
}
//--------------------------------
jfThreadEvent::jfThreadEvent(const jfThreadEvent &event):wxEvent(event) {
  int1 = event.int1;
  int2 = event.int2;
  option = event.option;
  string1 = event.string1;
  string2 = event.string2;
  ferr = event.ferr;
  ract = event.ract;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data
//--------------------------------
bool jfThreadEvent::GetOption() const { return option; }
//--------------------------------
int jfThreadEvent::GetInt1() const { return int1; }
//--------------------------------
int jfThreadEvent::GetInt2() const { return int2; }
//--------------------------------
wxString jfThreadEvent::GetString1() const { return string1; }
//--------------------------------
wxString jfThreadEvent::GetString2() const { return string2; }
//--------------------------------
jfDown::FetchResult jfThreadEvent::GetFetchResult() const { return ferr; }
//--------------------------------
jfDown::ResultAction jfThreadEvent::GetResultAction() const { return ract; }
//++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
void jfThreadEvent::SetOption(bool inval) { option = inval; }
//--------------------------------
void jfThreadEvent::SetInt1(int inval) { int1 = inval; }
//--------------------------------
void jfThreadEvent::SetInt2(int inval) { int2 = inval; }
//--------------------------------
void jfThreadEvent::SetInts(int inval1, int inval2) {
  int1 = inval1;
  int2 = inval2;
}
//--------------------------------
void jfThreadEvent::SetString1(const wxString& instring) { string1 = instring; }
//--------------------------------
void jfThreadEvent::SetString2(const wxString& instring) { string2 = instring; }
//--------------------------------
void jfThreadEvent::SetStrings(const wxString& instring1,const wxString& instring2) {
  string1 = instring1;
  string2 = instring2;
}
//--------------------------------
void jfThreadEvent::SetError(jfDown::FetchResult inerr) {
  ferr = inerr;
  string1 = jfDown::FetchResult2String(ferr);
}
//--------------------------------
void jfThreadEvent::SetResAction(jfDown::ResultAction in_ract) {
  ract = in_ract;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++
wxEvent* jfThreadEvent::Clone() const {
  return new jfThreadEvent(*this);
}
//****************************************************************************
