//****************************************************************************
// Name:        down_data.h
// Author :     John Q Metro
// Purpose :    Enumerations and types to help for downloading
// Created:     December 4, 2013
// Updated:     August 11, 2015
//****************************************************************************
#ifndef DOWN_DATA_H_INCLUDED
#define DOWN_DATA_H_INCLUDED
#endif // DOWN_DATA_H_INCLUDED
//----------------------------------

#include <wx/object.h>
#include <wx/string.h>
#include <wx/thread.h>
#include <wx/window.h>
//**************************************************************************
// a namespace to store some enums and basic download types
namespace jfDown {

  // an enumerated type for the status of a download
  enum DownStatus { NOTSTARTED, FETCHING, DONE_OK, DONE_FAILED};

  // an enumerated type for the result of a download
  enum FetchResult {
    FETCHED,    // got everything okay
    TIMEDOUT,   // timed out
    REDIRECTED, // redirected
    MISSING,    // 404 not found or the equivalent
    FAILED,     // misc faliure that could be temporary
    BLOCKED,    // failed, no point in retrying
    ABORTED,    // user requested download halt
    PARSE_ERROR,// parsing failed to work
    SSL_ERROR   // added just in case
  };

  // download status to string
  wxString DownStatus2String(const DownStatus& inval);
  // fetch result to string
  wxString FetchResult2String(const FetchResult& inval);

  // Retry Constants
  extern const size_t minor_retries;
  extern const size_t major_retries;
  extern const size_t timeout;
  size_t SleepPause();

  // an enumerated type for the pauser
  enum PauseStatus {WORKING, PAUSING, PAUSED, STOPPING, STOPPED};

  // an enumated type for what to do with a result
  enum ResultAction { PROCEED, HALT, SKIP, FAIL};
}
//==========================================================================
// simpler replacement for the old jfPauser
class jfNewPauser : public wxObject {
  public:
    // constructors and setting data
    jfNewPauser();
    jfNewPauser(wxWindow* target_in);
    bool SetTarget(wxWindow* new_target);
    bool SetOption(bool value);
    // external methods called to control the thread
    bool Pause();
    bool Resume();
    bool Stop();
    bool SpecialStop();
    // thread calls to pause or get stop info
    bool PauseHaltCheck();
    bool SpecialStopCheck();
  protected:
    // sending a pause message
    void SendPauseSignal();
    void SendResumeSignal();
    // mutexes
    mutable wxMutex statlock;
    wxMutex pauselock;
    // flags
    jfDown::PauseStatus status;
    bool special_stop;
    bool which_option;
    // message target
    wxWindow* msg_target;
};
//=======================================================================
// a simpler replacement for jfThreadParams
class jfDownloadParams : public wxObject {
  public:
    // data
    jfNewPauser* pauselink;
    wxWindow* msg_target;
    wxString incookie;
    size_t fetchlimit;
    bool pageskip;
    bool skipmissing;
    bool no_major_retry;
    // error functions
    bool (*CheckMissing)(const wxString*);
    bool (*CheckTruncated)(const wxString*);
    // constructor
    jfDownloadParams();
  protected:
};
//==========================================================================
// finally, for 'neatness', a custom event class
class jfThreadEvent : public wxEvent {
  public:
    // constructors
    jfThreadEvent(wxEventType commandType,int id = 0);
    jfThreadEvent(wxEventType commandType,int id, bool indata);
    jfThreadEvent(const jfThreadEvent &event);
    // getting data
    bool GetOption() const;
    int GetInt1() const;
    int GetInt2() const;
    wxString GetString1() const;
    wxString GetString2() const;
    jfDown::FetchResult GetFetchResult() const;
    jfDown::ResultAction GetResultAction() const;
    // setting data
    void SetOption(bool inval);
    void SetInt1(int inval);
    void SetInt2(int inval);
    void SetInts(int inval1, int inval2);
    void SetString1(const wxString& instring);
    void SetString2(const wxString& instring);
    void SetStrings(const wxString& instring1,const wxString& instring2);
    void SetError(jfDown::FetchResult inerr);
    void SetResAction(jfDown::ResultAction in_ract);
    // special
    wxEvent* Clone() const;
  protected:
    // internal data
    bool option;
    wxString string1, string2;
    int int1, int2;
    jfDown::FetchResult ferr;
    jfDown::ResultAction ract;
};
//-------------------------------------------------
// more thread event stuff
typedef void (wxEvtHandler::*jfThreadEventFunction)(jfThreadEvent&);

// custom event table entry
#define EVT_JF_THREAD(id,event_type,fn)                 \
	DECLARE_EVENT_TABLE_ENTRY( event_type, id, wxID_ANY,  \
	(wxObjectEventFunction)(wxEventFunction)              \
	(wxCommandEventFunction) wxStaticCastEvent(           \
	jfThreadEventFunction, &fn ), (wxObject*) NULL ),

//**************************************************************************
