/////////////////////////////////////////////////////////////////////////////
// Name:        expr_base.h
// Author :     John Q Metro
// Purpose :    Declares basic classes for expression elements
// Created:     26.12.06
// Updated:     April 29, 2009
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif
// ----------------------------------------------------------------------------

#ifndef JFBASEFILTER
#include "../filters/base/filterbase.h"
#endif
#ifndef BASECOLLECT_H_INCLUDED
#include "../structs/basecollect.h"
#endif // BASECOLLECT_H_INCLUDED


#ifndef JFEXPRTOKEN
#include "expr_token.h"
#endif

#define JFEXPRBASE
//*****************************************************************************
extern const wxString expr_operators;
extern const wxString expr_operands;
extern const wxString expr_precstring;
extern const wxString expr_scitems;
//=============================================================================
// a forward declaration
class jfExprElem;
//-----------------------------------------------------------------------------
// a special function for copying an element
jfExprElem* CopyElement(jfExprElem* source);
//----------------------------------------------------------------------------
// getting precedence
size_t GetPrec(const wxChar& intype);
//============================================================================
/* for name verification, the filter links can be treated as a tree. VerifyNames
 is used to probe this tree for problems. Only one branch is stored at a time,
 with backup up occurring when we reach the end of a branch.

 Since there are two ways to refer to a filter, there are two lists, local and
 global. No global expression is supposed to refer to a local filter, an object
 of this class stores the info required to detect that.

 Another error is not finding a filter, this is done in VerifyNames but we do
 not need a jfNameVerif object to detect this.

 For cicular references, we use a jfNameVerif object to hold a history list, and
 the VerfifyNames method searches for the current filter in the list. If it
 is there, this is an error. otherwise, we add the current filter to this list
 and call the Expression Filter's NameVerif method to check any embedded filter
 references.

 Note that if the current filter is *not* an Expression Filter, it is the end of
 a branch and all we need to do is check for local vs global and if the filter
 is there.
 */
// a special class used in verification
class jfNameVerif : public wxObject {
public:
	jfNameVerif();
	// some special error handling methods
	bool MakeChainString();
	wxString GetChainString() const;
	bool InGlobal() const;
	// looking for names
	bool FindName(const wxString& fname);
	bool FindName(const wxString& mapname, const wxString& fname);
	// name push and pop
	void PushGlobal(const wxString& mapname, const wxString& fname);
	bool PushLocal(const wxString& fname);
	bool PopName();
	// setting and getting the error
	bool InError() const;
	void SetLocalError(const wxString& lfiltname);
	void SetNotFoundError(const wxString& fname);
	void SetNotFoundError(const wxString& mapname, const wxString& fname);
	void SetCircularError(const wxString& fname);
	void SetCircularError(const wxString& mapname, const wxString& fname);
	wxString GetError() const;
	// the destructor
	~jfNameVerif();
protected:
    // local list
    wxArrayString* local_list;
    size_t local_count;
    // global list
    wxArrayString* global_list;
    size_t global_count;
    // error stuff
    wxString chainstring;
    wxString errormsg;
    bool inerror;
};
//=============================================================================
// the base type
class jfExprElem : public wxObject {
public:
    wxChar GetType() const;
    virtual bool IsOperand() const = 0;
    virtual wxString String() const = 0;
protected:
    // internal base data
    wxChar type;
};
//=============================================================================
// USABLE! --- For holding operators and also ()
class jfOperatorElem : public jfExprElem {
public:
    jfOperatorElem(wxChar typestr);
    jfOperatorElem(const jfOperatorElem& source);
    virtual bool IsOperand() const;
    virtual wxString String() const;
};
//============================================================================
// a default operand class
class jfOperandElem : public jfExprElem {
public:
    virtual bool IsOperand() const;
    virtual bool IsString() const = 0;
    bool IsValid() const;
protected:
    bool valid;
};
//============================================================================
// basic operand class for string matching
class jfStrElem : public jfOperandElem {
public:
    // info methods
    virtual bool IsString() const;
    wxString GetPattern() const;
    bool GetCasesen() const;
    bool GetTitle() const;
    virtual wxString String() const;
protected:
    wxString pattern;
    bool casesen;
    bool title;
};
//==========================================================================
// USABLE! --- Substring matching operand
class jfSubstringElem : public jfStrElem {
public:
    jfSubstringElem(const wxString& newpattern, bool newcs=false, bool istitle=false);
    jfSubstringElem(const jfSubstringElem& source);
    bool Match(const wxString& base) const;
};
//==========================================================================
// USABLE! --- Token string matching operand
class jfTokenMElem : public jfStrElem {
public:
    jfTokenMElem(const wxString& newpattern, bool newcs=false, bool istitle=false);
    jfTokenMElem(const jfTokenMElem& source);
    bool Match(const wxArrayString* base) const;
    ~jfTokenMElem();
protected:
    wxArrayString* tdata;
};
//=============================================================================
/* Operand class for embedded filter literals. We disallow embedded expression
 filters (they are redundant). This simplifies things quite a bit. */
class jfLiteralFilterElem : public jfOperandElem {
public:
    // constructors
    jfLiteralFilterElem(const wxString& ftype, const wxString& rawdata);
    jfLiteralFilterElem(const jfLiteralFilterElem& fsource);
    // public methods
    bool Match(const jfBasePD* testelem) const;
    // virtual methods that need to be defined
    virtual wxString String() const;
    virtual bool IsString() const;
    // destuctor
    ~jfLiteralFilterElem();
protected:
    wxString filttype, source;
    jfBaseFilter* thefilter;
};
//=============================================================================
/* the basic class for filter references, either to a local or global source.
 The filter is stored externally. Since the referenced filter can be an expression
 with filter references of it's own, we can get a problems with reference filters
 being renamed or deleted, or with circular references. VerifyNames is used to check
 that. Also, CopyLoad is used to copy the filter before actual searching starts */
class jfFilterElem : public jfOperandElem {
public:
    virtual bool IsString() const;
    bool Match(const jfBasePD* testelem) const;
    virtual bool IsExpr() const = 0;
    virtual bool VerfyNames(jfNameVerif* nameobj) = 0;
    virtual bool CopyLoad() = 0;
protected:
    jfBaseFilter* thefilter;
    wxString filter_name;
};
//=============================================================================
// a forward declaration
class jfFilterPointer;
//-----------------------------------------------------------------------------
// USABLE! --- Global Filter Reference
class jfGlobalFilterElem : public jfFilterElem {
public:
    jfGlobalFilterElem(const wxString& inmapname, const wxString& filtername);
    jfGlobalFilterElem(const jfGlobalFilterElem& source);
    virtual bool IsExpr() const;
    virtual bool VerfyNames(jfNameVerif* nameobj);
    virtual bool CopyLoad();
    virtual wxString String() const;
    ~jfGlobalFilterElem();
protected:
    wxString mapname;
    jfFilterPointer* c_pointer;
};
//=============================================================================
// USABLE ! ---- Local Filter Reference
class jfLocalFilterElem : public jfFilterElem {
public:
    jfLocalFilterElem(jfFilterMap* inlocalmap, const wxString& filtername);
    jfLocalFilterElem(const jfLocalFilterElem& source);
    virtual bool IsExpr() const;
    virtual bool VerfyNames(jfNameVerif* nameobj);
    virtual bool CopyLoad();
    virtual wxString String() const;
    ~jfLocalFilterElem();
protected:
    jfFilterMap* localmap;
};

//****************************************************************************
