/////////////////////////////////////////////////////////////////////////////
// Name:        expr_token.cpp
// Author :     John Q Metro
// Purpose :    Defines functions and constants for tokenizing
// Created:     04.01.07
// Updated:     10.01.07
/////////////////////////////////////////////////////////////////////////////

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------
#include <wx/tokenzr.h>
//----------------------------------------------------------------------------
#ifndef JFEXPRTOKEN
#include "expr_token.h"
#endif
#ifndef UTILS1_H_INCLUDED
#include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
//*****************************************************************************
// ascii delimiter constants
const wxString cc_ws = wxT(" \t\n\r\v\f");
// every symbol except _, *, and ?
const wxString cc_ascsym = wxT("!\"#$%&\'()+,-./:;<=>@[\\]^`{|}~");
// the match chars, usually delimiters but sometimes with special uses
const wxString cc_mdelim = wxT("*\?");
// latin 1 delimters
const wxString cc_latin1 = wxT("¡¢£¤¥¦§¨©ª«¬­®¯°±²³´µ¶¸¹º»¼½¾¿×÷");
//===========================================================================
// unicode delimiters,
#if wxUSE_UNICODE == 1
// greek semicolon and ano teleia
const wxString cc_greek = L"\u037E\u0387";
// armenian full stop and hyphen
const wxString cc_armenian = L"\u0589\u058A";
// hebrew maqaf,paseq,sof pasuq,nun hafukha, geresh, gershayim
const wxString cc_hebrew = L"\u05BE\u05C0\u05C3\u05C6\u05F3\u05F4";
// arabic comma, date separator, semicolon, triple dot, question mark, percent sign, full stop
const wxString cc_arab = L"\u060C\u060D\u061B\u061E\u061F\u066A\u06D4";
// devanagari danda, double danda
const wxString cc_devanag = L"\u0964\u0965";
// thai paiyan noi, bhat, mai yamok, fongman, angkhankhu, khomut
const wxString cc_thai = L"\u0E2F\u0E3F\u0E46\u0E4F\u0E5A\u0E5B";
// ethiopic
const wxString cc_ethiopic = L"\u1360\u1361\u1362\u1363\u1364\u1365\u1366\u1367\u1368";
// unicode spaces, U+2000 to U+200B
const wxString cc_unispaces = L"\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u200B";
// dashes and hyphens
const wxString cc_unidash = L"\u2010\u2012\u2013\u2014\u2015";
// unicode quotation marks
const wxString cc_uniquot = L"\u2018\u2019\u201A\u201B\u201C\u201D\u201E\u201F";
// extra bullets and dots
const wxString cc_unibdot = L"\u2020\u2021\u2022\u2023";
// more...
const wxString cc_moredelims = L"\u2026\u2027\u2028\u2030\u2031\u2032\u2033\u2039\u203A";
// more...
const wxString cc_moredelims2 = L"\u203B\u203C\u203D\u2047\u2048\u2049\u204A\u2052";
// currency symbols
const wxString cc_currsymb1 = L"\u20A0\u20A1\u20A2\u20A3\u20A4\u20A5\u20A6\u20A7\u20A8";
const wxString cc_currsymb2 = L"\u20A9\u20AA\u20AB\u20AC\u20AD\u20AE\u20AF\u20B0\u20B1";
const wxString cc_currsymb3 = L"\u20B2\u20B3\u20B4\u20B5";
#else // ANSI
const wxString cc_greek;
const wxString cc_armenian;
const wxString cc_hebrew;
const wxString cc_arab;
const wxString cc_devanag:
const wxString cc_thai;
const wxString cc_ethiopic;
const wxString cc_unispaces;
const wxString cc_unidash;
const wxString cc_uniquot;
const wxString cc_unibdot;
const wxString cc_moredelims;
const wxString cc_moredelims2;
const wxString cc_currsymb1;
const wxString cc_currsymb2;
const wxString cc_currsymb3;
#endif // Unicode/ANSI
//=============================================================================
// the global delimiters
const wxString cc_mlatdelims = cc_ws + cc_ascsym + cc_latin1;
const wxString cc_unidelims = cc_greek+cc_armenian+cc_hebrew+cc_arab+cc_devanag
+cc_thai+cc_ethiopic+cc_unispaces+cc_unidash+cc_uniquot+cc_unibdot+
cc_moredelims+cc_moredelims2+cc_currsymb1+cc_currsymb2+cc_currsymb3;
const wxString cc_mostdelims = cc_mlatdelims + cc_unidelims;
const wxString cc_alldelims = cc_mostdelims + cc_mdelim;
//===========================================================================
// tokenizes a string into a string array
wxArrayString* TokenizeString(wxString source, bool incmatch) {
	// local variables
	wxArrayString* result;
	wxStringTokenizer* toker;
	wxString tbuffer;
	// massaging the source string to simplify it
	Dtrim(source);
	source.Replace(wxT("*?"),wxT("?*"));
	source.Replace(wxT("**"),wxT("*"));
	// preparing for tokenizing
	result = new wxArrayString();
	if (incmatch) toker = new wxStringTokenizer(source,cc_alldelims,wxTOKEN_STRTOK);
	else toker = new wxStringTokenizer(source,cc_mostdelims,wxTOKEN_STRTOK);
	// next up is the tokenizing loop
	while (toker->HasMoreTokens()) {
		tbuffer = toker->GetNextToken();
		result->Add(tbuffer);
	}
	// now done, we clear up and return
	delete toker;
	return result;
}

//*****************************************************************************
