/*****************************************************************************
Name    :   searchcore.cpp
Author  :   John Q Metro
Purpose :   Declares a base class containing search info and parameters
Created :   May 4, 2009
Updated :   August 22, 2012
******************************************************************************/
#ifndef SEARCHCORE_H_INCLUDED
  #include "searchcore.h"
#endif // SEARCHCORE_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface/interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
#ifndef HTML_DISPLAY_H_INCLUDED
  #include "../../interface/display/html_display.h"
#endif // HTML_DISPLAY_H_INCLUDED
/*****************************************************************************/
// the constructor, just makes sure links are NULL
jfSearchCore::jfSearchCore() {
  def_filtermap = NULL;
  local_map = NULL;
  categories = NULL;
  cat_results = NULL;
  skel_link = NULL;
  local_skel = false;
  catindex = 0;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// checks the startup set for validity
bool jfSearchCore::Check() const {
  if (categories==NULL) return false;
  if (def_filtermap==NULL) return false;
  if (cat_results==NULL) return false;
  size_t gc = categories->GetCount();
  if (!(cat_results->CheckRCounts(categories->UsesDefault(),gc))) return false;
  if (local_map==NULL) return false;
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// makes a set of empty maps and categories
void jfSearchCore::MakeEmpty() {
  if (def_filtermap==NULL) def_filtermap = new jfFilterMap();
  if (local_map==NULL) local_map = new jfFilterMap();
  if (categories==NULL) categories = new jfCategories();
  categories->SetLocalMapPtr(local_map);
}
//---------------------------------------------------------------------
size_t jfSearchCore::GetPagelimit() const {
  return pagelimit;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy load (called just before search)
void jfSearchCore::CopyLoad() {
  // checks
  assert(local_map!=NULL);
  assert(def_filtermap!=NULL);
  assert(categories!=NULL);
  // calling CopyLoad on subparts
  def_filtermap->CopyLoad();
  categories->CopyLoad();
}
//--------------------------------------------------------------
bool jfSearchCore::VerifyNames(wxString&outmsg) {
  // constants
  const wxString fname = wxT("jfSearchCore::VerifyNames");
  // checks
  assert(local_map!=NULL);
  assert(def_filtermap!=NULL);
  assert(categories!=NULL);
  // variables
  wxString omsg;
  bool rval;
  // calling CopyLoad on subparts
  /**/jfXLogMessageB(fname,1,def_filtermap==NULL);
  /**/jfXLogMessageS(fname,2,def_filtermap->GetCount());
  rval = def_filtermap->VerifyNames(omsg,true);
  if (!rval) {
    outmsg = omsg + wxT(" in included filters.");
    return false;
  }
  /**/jfXLogMessage(fname,3);
  // calling CopyLoad on subparts
  rval = categories->VerifyNames(omsg);
  if (!rval) {
    outmsg = omsg + wxT(" in result categories.");
    /**/jfXLogMessage(fname,4,outmsg);
    return false;
  }
  else return true;
}
//--------------------------------------------------------------------
// initializes/or writes logging for pair filters
void jfSearchCore::PairFilterLogging(bool write) {
  // checks
  assert(local_map!=NULL);
  assert(def_filtermap!=NULL);
  assert(categories!=NULL);
  // calling CopyLoad on subparts
  local_map->PairFilterLogging(write);
  def_filtermap->PairFilterLogging(write);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// dispatches the event containing result data (to the organizer)
void jfSearchCore::DispatchResults(size_t stotal,jfResultUnitVector* inresult) {
  const wxString fname = wxT("@@@@ jfSearchCore::DispatchResults");
  wxCommandEvent outevent(jf_RESORG_PAGEDONE_EVENT,jfID_SRCCAT_PARSER);
  // checking
  jfXLogMessage(fname,1);
  assert(Check());
  assert(inresult!=NULL);
  assert(stotal>=(inresult->size()));
  assert(cat_results!=NULL);
  assert((inresult->size())>0);
  /**/jfXLogMessageS(fname,2,stotal);
  // setting the value of the event
  outevent.SetInt(stotal);
  outevent.SetClientData(inresult);
  /**/jfXLogMessage(fname,3);
  // we dispatch the result to the organizer
  wxPostEvent(cat_results,outevent);
  /**/jfXLogMessage(fname,4);
}
//--------------------------------------------------------------------
// sends a message containing items processed with no actual result
void jfSearchCore::DispatchCount(size_t icount) {
  const wxString fname = wxT("jfSearchCore::DispatchResults");
  wxCommandEvent outevent(jf_PARSER_ITEMSDONE_EVENT,jfID_SRCCAT_PARSER);
  // checking
  if (icount!=0) {
    outevent.SetInt(icount);
    wxPostEvent(cat_results,outevent);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/* if we do not use the default (leftover) result category, we return
TRUE if the paramster has no category marks */
bool jfSearchCore::NoCatTest(const jfBasePD* testee) const {
  assert(testee!=NULL);
  assert(categories!=NULL);
  if (!(categories->UsesDefault())) {
    if (testee->mark.HasMark()) return false;
    else return true;
  }
  else return false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfSearchCore::ClearLocalSkeleton() {
  if (local_skel) {
    assert(skel_link!=NULL);
    delete skel_link;
    skel_link = NULL;
    local_skel = false;
  }
}
//-------------------------------------------------------------------------
jfSkeletonBase* jfSearchCore::GetLocalSkeleton() {
  return skel_link;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// loading up data...
void jfSearchCore::LoadCoreValues(jfSkeletonParser* inparser, size_t result_categ) const {
  // checks
  assert(inparser!=NULL);
  assert(Check());
  assert(categories->ChIndex(result_categ));
  // we start to load values
  inparser->AddText(wxT("SEARCH_NAME"),name);
  inparser->AddText(wxT("SEARCH_DESCRIPTION"),description);
  inparser->AddUInt(wxT("SEARCH_ID"),num_id);
  // result category data
  if (result_categ==32) {
    inparser->AddText(wxT("RCATEG_NAME"),categories->GetName());
    inparser->AddText(wxT("RCATEG_DESCRIPTION"),categories->GetDescription());
  }
  else {
    inparser->AddText(wxT("RCATEG_NAME"),categories->NameAtIndex(result_categ));
    inparser->AddText(wxT("RCATEG_DESCRIPTION"),categories->DescAtIndex(result_categ));
  }

}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  implemented virtual i/o method
//----------------------------------------------------------------------
bool jfSearchCore::AddRestToFile(wxTextFile* outfile) const {
  wxString buffer;
  // the usual checks
  if (outfile==NULL) return false;
  if (!outfile->IsOpened()) return false;
  // adding a flag
  buffer = Bool2Str(local_skel);
  buffer += wxT(';');
  if (local_skel) buffer += Bool2Str((skel_link->GetSkelType()==jfs_MULTIPLE));
  else buffer += wxT("false");
  outfile->AddLine(buffer);
  // appending the subparts
  if (local_skel) {
   if (!(skel_link->AddToFile(outfile))) return false;
  }
  if (!(local_map->AddToFile(outfile))) return false;
  if (!(def_filtermap->AddToFile(outfile))) return false;
  if (!(categories->AddToFile(outfile))) return false;
  // done
  return true;
}
//-------------------------------------------------------------------------
bool jfSearchCore::ReadRestFromFile(jfFileReader* infile) {
  const wxString fname = wxT("jfSearchCore::ReadRestFromFile");
  // local variables
  bool multiple;
  jfSkeletonSingle* temp1;
  jfSkeletonMultiple* temp2;
  // we read in a flag line for a local skeleton store
  jfXLogMessage(fname,1);
  assert(infile!=NULL);
  if (!infile->ReadParseLine(2,fname)) return false;
  if (!infile->lp.BoolVal(0,local_skel)) return infile->BuildError(wxT("Not a Boolean value!"));
  if (!infile->lp.BoolVal(1,multiple)) return infile->BuildError(wxT("Not a Boolean value!"));
  jfXLogMessage(fname,2);
  // clearing out old skeleton data
  if (skel_link != NULL ) {
    jfXLogMessage(fname,3);
    delete skel_link;
    skel_link = NULL;
  }
  jfXLogMessage(fname,4);
  // next, we do read in the store if need be
  if (local_skel) {
    // we read in the skeleton store
    if (!multiple) {
      temp1 = new jfSkeletonSingle();
      if (!temp1->GetFromFile(infile)) {
        delete temp1;
        return false;
      }
      skel_link = temp1;
    }
    else {
      temp2 = new jfSkeletonMultiple();
      if (!temp2->GetFromFile(infile)) {
        delete temp2;
        return false;
      }
      skel_link = temp2;
    }
  }
  jfXLogMessage(fname,5);
  // local filters
  // creating the local filter map
  if (local_map!=NULL) delete local_map;
  local_map = new jfFilterMap();
  jfXLogMessage(fname,6);
  local_map->SetMapPtr(local_map);
  if (!local_map->GetFromFile(infile)) return false;
  jfXLogMessage(fname,7);
  // built-in filters
  if (def_filtermap!=NULL) delete def_filtermap;
  jfXLogMessage(fname,8);
  // the built-in filters
  def_filtermap = new jfFilterMap();
  def_filtermap->SetMapPtr(local_map);
  if (!def_filtermap->GetFromFile(infile)) return false;
  jfXLogMessage(fname,9);
  // the categories
  jfXLogMessage(fname,10);
  if (categories!=NULL) delete categories;
  categories = new jfCategories();
  categories->SetLocalMapPtr(local_map);
  if (!categories->GetFromFile(infile)) return false;
  jfXLogMessage(fname,11);
  return true;
}
/*****************************************************************************/
