/******************************************************************************
// Name:        typedcollect.h
// Author :     John Q Metro
// Purpose :    Declares a templated vector wrapper for descendants of
//              jfBasePD
// Created:     May 7, 2009
// Updated:     July 1, 2015 (discard when writing html file)
******************************************************************************/
#ifndef TYPEDCOLLECT_H_INCLUDED
#define TYPEDCOLLECT_H_INCLUDED
#endif // TYPEDCOLLECT_H_INCLUDED
//-----------------------------------------------------------------------------
#ifndef ITEMCOLLECT_H_INCLUDED
  #include "itemcollect.h"
#endif // ITEMCOLLECT_H_INCLUDED
#include <vector>
#ifndef LOGGING_H_INCLUDED
  #include "../utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/
// class Q must be a descendant of jfBasePD
template <class Q>
class jfTypedCollection : public jfUrlItemCollection {
  public:
	typedef typename std::vector<Q*>::const_iterator q_const_iterator;
	// constructor
    jfTypedCollection();
  // extra methods
    size_t Append(Q* initem);
    const Q* GetItem(const size_t& dindex) const;
    const Q* GetCurrentC() const;
    Q* GetCurrent();
  // virtual methods, now defined here
    virtual bool NextIndex();
    virtual bool BackIndex();
    virtual void ResetIndex();
    virtual bool ToIndex(const wxInt32& newindex);
    virtual size_t CountRCategory(size_t which) const;
    virtual wxString PercRCategory(size_t which) const;
    // extracting info at the specified index
    virtual wxString NameAtIndex(size_t i_index) const;
    virtual wxString DescriptionAtIndex(size_t i_index) const;
    virtual wxString UrlAtIndex(size_t i_index) const;
    virtual size_t IdAtIndex(size_t i_index) const;
    // setting info at the specified index
    virtual bool SetDiscardAtIndex(size_t i_index, bool setval);
    // some more abstract methods
    virtual wxArrayString* GetNameList() const;
    virtual wxArrayString* GetDescriptionList() const;
    virtual wxArrayString* GetTypeList() const;
    virtual bool DeleteAtIndex(const size_t& dindex);
    // clearing out stuff (for destruction!)
    void ClearContents(bool destroy_too);
  protected:
    // appends page_results onto maindata
    size_t AppendPageResults();
    // general post page result method
    size_t PostPageProcessing();
    //  implemented virtual i/o method
    virtual bool AddRestToFile(wxTextFile* outfile) const;
    virtual bool ReadRestFromFile(jfFileReader* infile);
    virtual bool ToHTMLList(jfHtmlParams* indata, bool single) const;

    // the data
    std::vector<Q*> mainlist;
};
//==============================================================================
// constructor
template <class Q>
jfTypedCollection< Q >::jfTypedCollection():jfUrlItemCollection() {}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// appending an item
template <class Q>
size_t jfTypedCollection< Q >::Append(Q* initem) {
  assert(initem!=NULL);
  mainlist.push_back(initem);
  item_count++;
  return item_count;
}
//-----------------------------------------------------------------------------
// getting a const item
template <class Q>
const Q* jfTypedCollection< Q >::GetItem(const size_t& dindex) const {
  // we first do the usual checks
  assert(dindex>=0);
  assert(dindex<item_count);
  return mainlist[dindex];
}
//-----------------------------------------------------------------------------
// getting a const item at the current index
template <class Q>
const Q* jfTypedCollection< Q >::GetCurrentC() const {
  if (curr_item==NULL) return NULL;
  else return dynamic_cast<const Q*>(curr_item);
}
//-----------------------------------------------------------------------------
// getting a modifiable item at the current index
template <class Q>
Q* jfTypedCollection< Q >::GetCurrent() {
  if (curr_item==NULL) return NULL;
  else return dynamic_cast<Q*>(curr_item);
}
//===============================================================================
// virtual methods, now defined here
//-----------------------------------------------------------------------------
// incrementing the index
template <class Q>
bool jfTypedCollection< Q >::NextIndex() {
  // already off the end
  if (item_index==-2) return false;
  // going off the end
  else if (item_index==(item_count-1)) {
    item_index = -2;
    curr_item = NULL;
    return false;
  }
  // going forward
  else {
    item_index++;
    curr_item = mainlist[item_index];
    return true;
  }
}
//-----------------------------------------------------------------------------
// decrementing the index
template <class Q>
bool jfTypedCollection< Q >::BackIndex() {
  // already before the start
  if (item_index==-1) return false;
  // going off the start
  if (item_index==0) {
    item_index = -1;
    curr_item = NULL;
    return false;
  }
  // moving onto the end
  if (item_index==-2) item_index = (item_count-1);
  else item_index--;
  if (item_index!=-1) {
    curr_item = mainlist[item_index];
    return true;
  }
  else return false;
}
//------------------------------------------------------------------------------
// sets the index to -1
template <class Q>
void jfTypedCollection< Q >::ResetIndex() {
  item_index = -1;
  curr_item = NULL;
}
//------------------------------------------------------------------------------
// we move to a particular index
template <class Q>
bool jfTypedCollection< Q >::ToIndex(const wxInt32& newindex) {
  if (newindex<-2) return false;
  if (newindex>=item_count) return false;
  item_index = newindex;
  curr_item = mainlist[item_index];
  return true;
}
//-------------------------------------------------------------------------------
/* counts how many items are marked as belonging to a particular result
category. 32 counts as no category at all */
template <class Q>
size_t jfTypedCollection< Q >::CountRCategory(size_t which) const {
  assert(which<33);
  // variables
  size_t rcount = 0;
  size_t lcount;
  // the main loop
  for (lcount=0;lcount<item_count;lcount++) {
    if (which==32) {
      if (!(mainlist[lcount]->mark).HasMark()) rcount++;
    }
    else {
      if ((mainlist[lcount]->mark).IsMarked(which)) rcount++;
    }
  }
  // done
  return rcount;
}
//----------------------------------------------------------------------------
// outputs the percentage of the total fetched that beglong to a particular result category
template <class Q>
wxString jfTypedCollection< Q >::PercRCategory(size_t which) const {
  // variables
  size_t rtotal;
  wxString resval;
  double pval;
  // we start
  assert(which<33);
  if (grand_total[0]==0) resval = wxT("0.00%");
  else {
    rtotal = CountRCategory(which);
    pval = (double(rtotal))/(double(grand_total[0]));
    resval = ToPercent(pval,2);
  }
  // done
  return resval;
}
//===============================================================================
// extracting info at the specified index
//------------------------------------------------------------------------------
template <class Q>
wxString jfTypedCollection< Q >::NameAtIndex(size_t i_index) const {
  if (i_index>=item_count) return wxEmptyString;
  else return mainlist[i_index]->GetName();
}
//------------------------------------------------------------------------------
template <class Q>
wxString jfTypedCollection< Q >::DescriptionAtIndex(size_t i_index) const {
  if (i_index>=item_count) return wxEmptyString;
  else return mainlist[i_index]->GetDescription();
}
//------------------------------------------------------------------------------
template <class Q>
wxString jfTypedCollection< Q >::UrlAtIndex(size_t i_index) const {
  if (i_index>=item_count) return wxEmptyString;
  else return mainlist[i_index]->GetUrl();
}
//------------------------------------------------------------------------------
template <class Q>
size_t jfTypedCollection< Q >::IdAtIndex(size_t i_index) const {
  if (i_index>=item_count) return 0;
  else return mainlist[i_index]->GetID();
}
//===================================================================
    // setting info at the specified index
template <class Q>
bool jfTypedCollection< Q >::SetDiscardAtIndex(size_t i_index, bool setval) {
  if (i_index>=item_count) false;
  mainlist[i_index]->discard = setval;
  return true;
}
//===============================================================================
// some more abstract methods
//-----------------------------------------------------------------------------
// return a lsit of all the names
template <class Q>
wxArrayString* jfTypedCollection< Q >::GetNameList() const {
  // the variables
  size_t loopc,lmax;
  wxArrayString* result;
  // we start the preparation
  lmax = item_count-1;
  if (item_count==0) return NULL;
  result = new wxArrayString();
  // the loop
  for (loopc=0;loopc<lmax;loopc++) {
    result->Add(mainlist[loopc]->GetName());
  }
  // done
  return result;
}
//-----------------------------------------------------------------------------
template <class Q>
wxArrayString* jfTypedCollection< Q >::GetDescriptionList() const {
  // the variables
  size_t loopc,lmax;
  wxArrayString* result;
  // we start the preparation
  lmax = item_count-1;
  if (item_count==0) return NULL;
  result = new wxArrayString();
  // the loop
  for (loopc=0;loopc<lmax;loopc++) {
    result->Add(mainlist[loopc]->GetDescription());
  }
  // done
  return result;
}
//-----------------------------------------------------------------------------
template <class Q>
wxArrayString* jfTypedCollection< Q >::GetTypeList() const {
  // the variables
  size_t loopc,lmax;
  wxArrayString* result;
  // we start the preparation
  lmax = item_count-1;
  if (item_count==0) return NULL;
  result = new wxArrayString();
  // the loop
  for (loopc=0;loopc<lmax;loopc++) {
    result->Add(mainlist[loopc]->GetTypeID());
  }
  // done
  return result;
}
//-----------------------------------------------------------------------------
/* when deleting at an index, we keep the stored indexes the same unless the
stored index points towards the last item, in which case we move it to the
previosu index. The actual items will change, though */
template <class Q>
bool jfTypedCollection< Q >::DeleteAtIndex(const size_t& dindex) {
  // data values
  size_t nlast;
  // first, we do the usual checks
  if (dindex<0) return false;
  if (dindex>=item_count) return false;
  // we delete the item
  delete mainlist[dindex];
  mainlist[dindex] = NULL;
  // we we shorten the list
  mainlist.erase(mainlist.begin()+dindex);
  nlast = item_count-2;
  // a special case... no item left!
  if (item_count==1) {
    item_count=0;
    item_index = -1;
    curr_item = NULL;
    backup_index = -1;
    backup_item = NULL;
    return true;
  }
  // next, we redo the indexes
  // if the index is the last, we leave it as the last
  if (item_index==(item_count-1)) {
    item_index = nlast;
    curr_item = mainlist[nlast];
  }
  if (backup_index==(item_count-1)) {
    backup_index = nlast;
    backup_item = mainlist[nlast];
  }
  // otherwise, we keep the actual index the same, but change the ite,
  if (item_index>=0) curr_item = mainlist[item_index];
  if (backup_index>=0) backup_item = mainlist[item_index];
  // finishing off
  item_count--;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// clearing out stuff (for destruction!)
template <class Q>
void jfTypedCollection< Q >::ClearContents(bool destroy_too) {
  // variables
  size_t rloop;
  // if we destroy, we do that first
  if (destroy_too) {
    for (rloop=0;rloop<item_count;rloop++) {
      delete mainlist[rloop];
      mainlist[rloop] = NULL;
    }
  }
  // getting rid of the list
  mainlist.clear();
  //getting rid of the indexes
  item_index = -1;
  curr_item = NULL;
  backup_index = -1;
  backup_item = NULL;
  // resetting the counts
  item_count = 0;
  for (rloop=0;rloop<3;rloop++) {
    pagecount[rloop] = 0;
    grand_total[rloop] = 0;
  }
  // some final stuff
  pageindex = 0;
  if (page_results!=NULL) {
    delete page_results;
    page_results = NULL;
  }
  newpage_items = 0;
  pr_size = 0;
  pr_index = 0;
}
//============================================================================
// appends page_results onto maindata
template <class Q>
size_t jfTypedCollection< Q >::AppendPageResults() {
  // variables and constants
  const wxString fname = wxT("jfTypedCollection< Q >::AppendPageResults()");
  size_t loopc,res;
  Q* copyi;
  // special cases
  /**/jfXLogMessage(fname,1);
  if (page_results==NULL) return 0;
  /**/jfXLogMessage(fname,2);
  if (pr_size==0) {
    delete page_results;
    page_results = NULL;
    return 0;
  }
  /**/jfXLogMessageS(fname,3,pr_size);
  // general moving over
  mainlist.resize(item_count+pr_size);
  for (loopc=0;loopc<pr_size;loopc++) {
    copyi = dynamic_cast<Q*>((*page_results)[loopc]);
    mainlist[item_count+loopc] = copyi;
  }
  /**/jfXLogMessage(fname,4);
  // getting rid of the old
  delete page_results;
  res = pr_size;
  pr_index=-1;
  pr_size=0;
  page_results = NULL;
  /**/jfXLogMessage(fname,5);
  // finishing
  item_count+=res;
  return res;
}
//-------------------------------------------------------------------------
// general post page result method
template <class Q>
size_t jfTypedCollection< Q >::PostPageProcessing() {
  // constants
  const wxString fname = wxT("jfTypedCollection< Q >::PostPageProcessing()");
  // variables
  size_t loopc; Q* worki;
  // filtering and sorting the result
  FilterByList();
  CategorySort();
  /**/jfXLogMessage(fname,1);
  Condense();
  /**/jfXLogMessage(fname,2);
  // processing the summaries
  if (page_results!=NULL) {
    /**/jfXLogMessage(fname,3);
    for (loopc=0;loopc<pr_size;loopc++) {
      worki = dynamic_cast<Q*>((*page_results)[loopc]);
      worki->ProcessDescription();
    }
  }
  // sending a message about what we have
  DispatchPageResults();
  /**/jfXLogMessage(fname,4);
  // finally, adding what we have onto the main list
  return AppendPageResults();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  implemented virtual i/o method
//-----------------------------------------------------------------------
template <class Q>
bool jfTypedCollection< Q >::AddRestToFile(wxTextFile* outfile) const {
  size_t loopc;
  // the usual checks
  if (outfile==NULL) return false;
  if (!outfile->IsOpened()) return false;
  // next, we call a method that must be implemented by any non-abstract descendant
  if (!AddDelta(outfile)) return false;
  // writing the results loop
  for (loopc=0;loopc<item_count;loopc++) {
    if (!(mainlist[loopc]->AddToFile(outfile))) return false;
  }
  // done
  return true;
}
//---------------------------------------------------------------------------
template <class Q>
bool jfTypedCollection< Q >::ReadRestFromFile(jfFileReader* infile) {
  const wxString fname = wxT("jfTypedCollection< Q >::ReadRestFromFile");
  // local variables
  size_t loopc;
  Q* initem;
  // starting
  if (!ReadDelta(infile)) return false;
  /* item count is already read, we assume the line index is just before
  the first item */
  for (loopc=0;loopc<item_count;loopc++) {
    // we prepare to read the item
    initem = new Q();
    // reading the item
    if (!initem->GetFromFile(infile)) {
      delete initem;
      return false;
    }
    // adding the item to our list
    mainlist.push_back(initem);
  }
  // done
  return true;
}
//----------------------------------------------------------------------------
/* iterates over the items, producing a wxString contatining an HTML representation
suitable for an HTML file */
template <class Q>
bool jfTypedCollection< Q >::ToHTMLList(jfHtmlParams* indata, bool single) const {
  // local variables
  wxString result;
  q_const_iterator findex;
  bool use_it;
  size_t numresults;
  wxString tvalue;
  // we start
  findex = mainlist.begin();
  numresults = 0;
  // the main loop
  while (findex<mainlist.end()) {
    // we check if we use the current item
    if (indata->GetResCat()==32) {
      use_it = (!(*findex)->mark.HasMark());
    }
    else {
      use_it = (*findex)->mark.IsMarked(indata->GetResCat());
    }
    if (use_it) use_it = !((*findex)->discard);
    // we only do the next if use_it is true
    if (use_it) {
      /* we add a separator first (since there is no way to tell if there
      are any following items */
      if (numresults!=0) {
        tvalue = (single)?(indata->Single()->item_separator):(indata->Multiple()->item_separator);
        result = indata->MakeResult(tvalue);
        indata->outfile->AddLine(result);
      }
      // getting and writing the html data for the item
      (*findex)->LoadValues(indata->parse);
      tvalue = (single)?(indata->Single()->item_skel):(indata->Multiple()->item_skel);
      result = indata->MakeResult(tvalue);
      indata->outfile->AddLine(result);
      // index increments
      numresults++;
    }
    findex++;
  }
  // done
  return true;
}
/*****************************************************************************/
