/******************************************************************************
Name    :   expr_editor.h
Author  :   John Q Metro
Purpose :   Editors for jfExpressionFiler, and also categories
Created :   March 26, 2009
Updated :   August 22, 2012
******************************************************************************/
#ifndef EXPR_EDITOR_H_INCLUDED
  #include "expr_editor.h"
#endif // EXPR_EDITOR_H_INCLUDED
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef FILTER_WIDGETS_H_INCLUDED
  #include "filter_picker.h"
#endif // FILTER_WIDGETS_H_INCLUDED
#ifndef UTILS2_H_INCLUDED
  #include "../../../core/utils/utils2.h"
#endif // UTILS2_H_INCLUDED
//****************************************************************************
jfExpressionEditor::jfExpressionEditor(wxWindow* parent, wxWindowID id,
        jfFilterMap* inmap, bool notinmap, const bool& multiline, bool in_global):wxPanel(parent,id) {
  // we start with checking the inmap
  assert(inmap!=NULL);
  local_fmap = inmap;
  error_code = 1;
  map_nomap = notinmap;
  isglobal = in_global;
  // sw start with button creation
  finsert_btn = new wxButton(this,jfID_EXPR_EDITOR_PICK,wxT("Pick Filter"));
  fcheck_btn = new wxButton(this,jfID_EXPR_EDITOR_CHECK,wxT("Check"));
  // creating the top sizer, which is always horizontal
  topsizer = new wxBoxSizer(wxHORIZONTAL);
  // what we do next depends on whether the editor is multiline or not
  if (multiline) {
    editcore = new wxTextCtrl(this,jfID_EXPR_EDITOR_EDIT,wxT(""),
      wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_BESTWRAP);
    // we start arranging
    rightsizer = new wxBoxSizer(wxVERTICAL);
    rightsizer->Add(finsert_btn,0,wxALL ,3);
    rightsizer->Add(fcheck_btn,0,wxALL ,3);
    topsizer->Add(editcore,1,wxEXPAND|wxALL|wxALIGN_CENTER_VERTICAL,3);
    topsizer->Add(rightsizer,0,wxALL,1);
  }
  else {
    editcore = new wxTextCtrl(this,jfID_EXPR_EDITOR_EDIT,wxT(""));
    // we start arranging
    topsizer->Add(editcore,1,wxEXPAND|wxALL|wxALIGN_CENTER_VERTICAL,3);
    topsizer->Add(finsert_btn,0,wxALL ,3);
    topsizer->Add(fcheck_btn,0,wxALL ,3);
  }
  // done with creation and sizing
  SetSizer(topsizer);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfExpressionEditor::PressPickFilter(wxCommandEvent& event) {
  // local variables
  jfFilterPickerDialog* pickerdialog;
  jfGlobalFilterPickerDialog* gpicker_dialog;
  long elength, iindex;
  wxString* insert_result;
  bool dialog_result;
  // we record the length and insertion point
  elength = editcore->GetLastPosition();
  iindex = editcore->GetInsertionPoint();
  // start preparations to show the dialog
  insert_result = new wxString();
  // the global only version
  if (isglobal) {
    gpicker_dialog = new jfGlobalFilterPickerDialog(this,-1,wxT("Pick a Global filter"),insert_result);
    dialog_result = ((gpicker_dialog->ShowModal())==wxID_OK);
    gpicker_dialog->Destroy();
  }
  else { // or the one for local as well
    pickerdialog = new jfFilterPickerDialog(this,-1,wxT("Pick a Local or Global filter"),local_fmap,insert_result);
    dialog_result = ((pickerdialog->ShowModal())==wxID_OK);
    pickerdialog->Destroy();
  }
  // showing
  if (dialog_result) {
    if (!(*insert_result).IsEmpty()) {
      // we insert the result in the editor
      editcore->SetInsertionPoint(iindex);
      editcore->WriteText((*insert_result));
    }
  }
  // cleaning up the results
  delete insert_result;
}
//--------------------------------------------------------------------------
void jfExpressionEditor::PressCheckFilter(wxCommandEvent& event) {
  // local variables
  const wxString rvalid = wxT("Expression Valid");
  const wxString nvalid = wxT("Expression Not Valid");
  wxString dstring, errstring;
  jfExpressionFilter *result_expression;
  wxMessageDialog* message;
  // starting
  result_expression = CheckFilter(errstring);
  if (result_expression==NULL) {
    // we are in error here
    message = new wxMessageDialog(this,errstring,nvalid,wxOK);
    message->ShowModal();
  }
  else {
    // no error
    if (result_expression->IsEmpty()) dstring = wxT("Empty expressions match everything.");
    else dstring = wxT("Everthing okay.");
    message = new wxMessageDialog(this,dstring,rvalid,wxOK);
    message->ShowModal();
  }
  // done
  delete result_expression;
  message->Destroy();
}
//--------------------------------------------------------------------------
void jfExpressionEditor::TextChanged(wxCommandEvent& event) {
  SetStatus(1);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfExpressionFilter* jfExpressionEditor::CheckFilter(wxString& outmessage) {
  // local variables
  wxString dstring;
  jfNameVerif* verifier;
  jfExpressionFilter *result_expression;
  // starting...
  dstring = editcore->GetValue();
  // empty expresssions
  if (dstring.IsEmpty()) {
    outmessage = wxT("Empty expressions match everything.");
    result_expression = new jfExpressionFilter();
    result_expression->EmptyFilter();
    result_expression->SetFiltermapLink(local_fmap);
    SetStatus(0);
    return result_expression;
  }
  // non-empty expressions
  else {
    result_expression = new jfExpressionFilter();
    result_expression->SetFiltermapLink(local_fmap);
    if (!result_expression->FromString(dstring) ) {
      outmessage = result_expression->parse_error;
      delete result_expression;
      SetStatus(2);
      return NULL;
    }
    // we check for validity
    verifier = new jfNameVerif();
    if (!(result_expression->VerifyNames(verifier,map_nomap))) {
      outmessage = verifier->GetError();
      delete result_expression;
      SetStatus(2);
      return NULL;
    }
    else {
      outmessage=wxT("No errors detected.");
      SetStatus(0);
      return result_expression;
    }
  }
}
//-----------------------------------------------------------------------------
/* This method treats in input string as an expression and reports on it's
validity */
bool jfExpressionEditor::CheckExpr(const wxString& inexpr,wxString& outmessage) const {
  // local variables
  jfNameVerif* verifier;
  jfExpressionFilter *result_expression;
  // starting...
  // empty expresssions
  if (inexpr.IsEmpty()) {
    outmessage = wxT("Empty expressions match everything.");
    return true;
  }
  // non-empty expressions
  else {
    result_expression = new jfExpressionFilter();
    result_expression->SetFiltermapLink(local_fmap);
    if (!result_expression->FromString(inexpr)) {
      outmessage = result_expression->parse_error;
      delete result_expression;
      return false;
    }
    // we check for validity
    verifier = new jfNameVerif();
    if (!(result_expression->VerifyNames(verifier,map_nomap))) {
      outmessage = verifier->GetError();
      delete result_expression;
      return false;
    }
    else {
      outmessage=wxT("No errors detected.");
      delete result_expression;
      return true;
    }
  }
}
//-----------------------------------------------------------------------------
// setting data
bool jfExpressionEditor::SetData(const wxString& inexpr,wxString& outmessage) {
  // getting the check result
  bool chkres = CheckExpr(inexpr,outmessage);
  editcore->ChangeValue(inexpr);
  if (chkres) SetStatus(0);
  else SetStatus(2);
  return chkres;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper
void jfExpressionEditor::SetStatus(size_t instatus) {
  assert(instatus<3);
  if (instatus==0) {
    editcore->SetForegroundColour(*wxGREEN);
    editcore->SetModified(false);
  }
  else if (instatus==1) {
    editcore->SetForegroundColour(wxNullColour);
    editcore->SetModified(true);
  }
  else {
    editcore->SetForegroundColour(*wxRED);
    editcore->SetModified(false);
  }
  error_code = instatus;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfExpressionEditor,wxPanel)
  EVT_BUTTON(jfID_EXPR_EDITOR_PICK,jfExpressionEditor::PressPickFilter)
  EVT_BUTTON(jfID_EXPR_EDITOR_CHECK,jfExpressionEditor::PressCheckFilter)
  EVT_TEXT(jfID_EXPR_EDITOR_EDIT,jfExpressionEditor::TextChanged)
END_EVENT_TABLE()
//****************************************************************************
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jfUrlFilEdit::jfUrlFilEdit(wxWindow* parent, wxWindowID id, bool side_arrangement):
        wxPanel(parent,id) {

  // we prepare the lists and thier labels
  // alternates
  alt_labl = new wxStaticText(this,-1,wxT("The url must match ONE of the patterns below:"));
  alt_incls = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_DONTWRAP);
  // excludes
  excl_labl = new wxStaticText(this,-1,wxT("The url must NOT match ANY of the patterns below:"));
  excludes = new wxTextCtrl(this,-1,wxEmptyString,wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_DONTWRAP);
  // next up: the checkboxes
  case_insensetive = new wxCheckBox(this,-1,wxT("Case insensetive matching"));
  excludes_only = new wxCheckBox(this,jfID_UFILTE_EOCHECK,wxT("This filter acts to exclude urls only"));
  https_counted = new wxCheckBox(this,-1,wxT("HTTPS as well as HTTP urls will be matched"));
  https_counted->SetValue(true);
  // saving the arrangement
  sidearr = side_arrangement;
  // start with sizers
  if (sidearr) {
    // the left column has everything except the exludes
    left_sizer = new wxBoxSizer(wxVERTICAL);
    left_sizer->Add(case_insensetive,0,wxALL,3);
    left_sizer->Add(https_counted,0,wxALL,3);
    left_sizer->Add(excludes_only,0,wxALL,3);
    left_sizer->AddSpacer(3);
    left_sizer->Add(alt_labl,0,wxALL,1);
    left_sizer->Add(alt_incls,1,wxALL|wxEXPAND,3);
    // the right column has the excludes
    right_sizer = new wxBoxSizer(wxVERTICAL);
    right_sizer->Add(excl_labl,0,wxALL,1);
    right_sizer->Add(excludes,1,wxALL|wxEXPAND,3);
    // putting the two sizers together
    totalsizer = new wxBoxSizer(wxHORIZONTAL);
    totalsizer->Add(left_sizer,1,wxALL|wxEXPAND,2);
    totalsizer->Add(right_sizer,1,wxALL|wxEXPAND,2);
  }
  else {
    // one column for everything
    left_sizer = NULL;
    right_sizer = NULL;
    totalsizer = new wxBoxSizer(wxVERTICAL);
    // we stack things in
    totalsizer->Add(case_insensetive,0,wxALL,3);
    totalsizer->Add(https_counted,0,wxALL,3);
    totalsizer->Add(excludes_only,0,wxALL,3);
    totalsizer->AddSpacer(3);
    totalsizer->Add(alt_labl,0,wxALL,1);
    totalsizer->Add(alt_incls,1,wxALL|wxEXPAND,3);
    totalsizer->Add(excl_labl,0,wxALL,1);
    totalsizer->Add(excludes,1,wxALL|wxEXPAND,3);
  }
  SetSizer(totalsizer);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
void jfUrlFilEdit::ExcludesOnlyChecked(wxCommandEvent& event) {
  if (excludes_only->IsChecked()) {
    alt_incls->Clear();
    alt_incls->Enable(false);
  }
  else {
    alt_incls->Enable(true);
  }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// return a filter with no name and description set, using the contents of the GUI
jfUrlFilter* jfUrlFilEdit::GetData() const {
  // local variables
  jfUrlFilter* result;
  wxArrayString* x1;
  // starting
  result = new jfUrlFilter();
  if (IsEmpty()) return result;
  // we set the boolean values first
  result->cimatch = (case_insensetive->IsChecked());
  result->https_incl = (https_counted->IsChecked());
  // setting the alternates
  x1 = TextCtrl2List(alt_incls);
  result->SetAlternates(x1);
  // seeting the excludes
  x1 = TextCtrl2List(excludes);
  result->SetExcludes(x1);
  // done
  return result;
}
//-----------------------------------------------------------------------------
// thw filter panel is empty is there is nothing other than whitespace in the wxTextCtrls
bool jfUrlFilEdit::IsEmpty() const {
  wxString buffer;
  bool t1,t2;
  buffer = alt_incls->GetValue();
  Dtrim(buffer);
  t1 = buffer.IsEmpty();
  buffer = excludes->GetValue();
  Dtrim(buffer);
  t2 = buffer.IsEmpty();
  return (t1 && t2);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// setting data
bool jfUrlFilEdit::SetData(const jfUrlFilter* indata) {
  // local variables
  wxArrayString* tval;
  bool tvaln;
  size_t loopc;
  // basic checks
  assert(indata!=NULL);
  // setting the checked items first
  case_insensetive->SetValue(indata->cimatch);
  https_counted->SetValue(indata->https_incl);
  // setting the alternates
  tval = indata->GetAlternates();
  tvaln = (tval!=NULL);
  if (tvaln) {
    excludes_only->SetValue(false);
    alt_incls->Enable(true);
    alt_incls->Clear();
    for (loopc=0;loopc<(tval->Count());loopc++) {
      alt_incls->AppendText((*tval)[loopc]);
      alt_incls->AppendText(wxT('\n'));
    }
    delete tval;
  }
  // setting the excludes
  tval = indata->GetExcludes();
  excludes->Clear();
  if (tval!=NULL) {
    for (loopc=0;loopc<(tval->Count());loopc++) {
      excludes->AppendText((*tval)[loopc]);
      excludes->AppendText(wxT('\n'));
    }
    delete tval;
  }
  if (!tvaln) {
    excludes_only->SetValue(true);
    alt_incls->Enable(false);
  }
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfUrlFilEdit,wxPanel)
  EVT_CHECKBOX(jfID_UFILTE_EOCHECK,jfUrlFilEdit::ExcludesOnlyChecked)
END_EVENT_TABLE()
//*****************************************************************************
