/******************************************************************************
Name    :   filter_picker.cpp
Author  :   John Q Metro
Purpose :   Defines some custom interface elements for a filter picker dialog.
Created :   March 25, 2009
Updated :   March 10, 2013 (column resizing for jfFilterPicker)
******************************************************************************/
#ifndef FILTER_WIDGETS_H_INCLUDED
  #include "filter_picker.h"
#endif // FILTER_WIDGETS_H_INCLUDED
#ifndef UTILS1_H_INCLUDED
  #include "../../core/utils/utils1.h"
#endif // UTILS1_H_INCLUDED
#ifndef JFILTERGLOBALS
  #include "../../../core/filters/base/filter_globals.h"
#endif
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED

#include <math.h>
//*****************************************************************************
// the constructor
jfFilterPickerBase::jfFilterPickerBase(wxWindow* parent,bool three_cols,
      wxWindowID id, const wxString& name):wxListCtrl(parent,id,
      wxDefaultPosition,wxDefaultSize,wxLC_REPORT | wxLC_SINGLE_SEL ,
      wxDefaultValidator,name) {
  // with the basic constructor done, we add the 3 columns
  InsertColumn(0,wxT("Filter Name"));
  InsertColumn(1,wxT("Filter Type"));
  if (three_cols) InsertColumn(2,wxT("Description"));
  // setting local variables
  fnames = NULL;
  ftypes = NULL;
  fdesc = NULL;
  loaded = false;
  three_columns = three_cols;
  old_selection = -1;
  select_index = -1;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// uses information from data the set the list
size_t jfFilterPickerBase::SetContents(const jfFilterMap* data, wxString inprefix) {
  size_t loopv, tcount;
  // some checks
  assert(data!=NULL);
  // clearing the old stuff
  DeleteAllItems();
  // a special case
  tcount = data->GetCount();
  prefix = inprefix;
  if (tcount==0) return 0;
  // extracting the lists
  fnames = data->GetNameList();
  if (three_columns) fdesc = data->GetDescriptionList();
  ftypes = data->GetTypeList();
  loaded = true;
  // now, do do the loading list
  for (loopv=0;loopv<tcount;loopv++) {
    InsertFLine(loopv);
  }
  // we no longer need the lists
  ClearLoaded();
  // setting the column width
  SetColumnWidth(0,-1);
  SetColumnWidth(1,-1);
  if (three_columns) SetColumnWidth(2,-1);
  // done
  return tcount;
}
//----------------------------------------------------------------------------
bool jfFilterPickerBase::ClearLoaded() {
  if (loaded) {
    fnames->Clear();
    delete fnames;
    fnames = NULL;
    if (three_columns) {
      fdesc->Clear();
      delete fdesc;
      fdesc = NULL;
    }
    ftypes->Clear();
    delete ftypes;
    ftypes = NULL;
    loaded = false;
    return true;
  }
  else return false;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFilterPickerBase::IsFilterSelected() const {
  return sloaded;
}
//---------------------------------------------------------------------------
bool jfFilterPickerBase::ChangeSelected(size_t nindex) {
  // if there is a selected item, we unselect it
  if (sloaded) Deselect();
  // we select
  return SelectNoEvent(nindex,true);
}
//----------------------------------------------------------------------------
long jfFilterPickerBase::SelectedIndex() const {
  if (sloaded) return select_index;
  else return -1;
}
//----------------------------------------------------------------
long jfFilterPickerBase::OldSelected() const {
  return old_selection;
}
//----------------------------------------------------------------------------
wxString jfFilterPickerBase::SelectedName() const {
  if (!sloaded) return wxT("");
  else return select_name;
}
//------------------------------------------------------------------------------
void jfFilterPickerBase::Deselect() {
  if (sloaded) SelectNoEvent(select_index,false);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// column sizing (alternative to pixel, uses proportions)
void jfFilterPickerBase::SetColWidth(size_t prop1, size_t prop2, size_t prop3) {
  // variables
  size_t propsum;
  double dprop1, dprop2,dprop3;
  int cwidth, ssize1, ssize2, ssize3;
  wxSize controlsize;
  // getting the control width
  controlsize = GetClientSize();
  cwidth = controlsize.GetWidth();
  // calculating the fractional proportions
  propsum = prop1 + prop2;
  if (three_columns) propsum += prop3;
  dprop1 = (double)prop1 / (double)propsum;
  dprop2 = (double)prop2 / (double)propsum;
  if (three_columns) {
    dprop3 = (double)prop3 / (double)propsum;
  }
  // getting the new integer width
  ssize1 = floor(dprop1*cwidth);
  ssize2 = floor(dprop2*cwidth);
  if (three_columns) ssize3 = floor(dprop3*cwidth);
  // applying the widths
  SetColumnWidth(0,ssize1);
  SetColumnWidth(1,ssize2);
  if (three_columns) SetColumnWidth(2,ssize3);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// events
void jfFilterPickerBase::OnItemSelect(wxListEvent& event) {
  if (!bypass) {
    // looking for the selected index
    bool atest = SetSelected();
    assert(atest);
    // we pass the event back up
    event.Skip();
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfFilterPickerBase::InsertFLine(const size_t& iindex) {
  wxString buffer;
  long newindex;
  // we do some checks first
  assert(loaded);
  assert(iindex<(fnames->GetCount()));
  // creating the new line and adding the name at the ame time
  buffer = fnames->Item(iindex);
  newindex = InsertItem(iindex,buffer);
  // adding type
  buffer = ftypes->Item(iindex);
  SetItem(newindex,1,buffer);
  // adding description
  if (three_columns) {
    buffer = fdesc->Item(iindex);
    SetItem(newindex,2,buffer);
  }
  // done
  return true;
}
//--------------------------------------------------------------------------
bool jfFilterPickerBase::SelectNoEvent(size_t index, bool issel) {
  bool atest;
  if (index>=GetItemCount()) return false;
  bypass = true;
  if (issel) {
    atest = SetItemState(index,wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    assert(atest);
    select_name = GetItemText(index);
    old_selection = select_index;
    select_index = index;
    sloaded = true;

  }
  else {
    atest = SetItemState(index,0,wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED);
    assert(atest);
    sloaded = false;
    old_selection = select_index;
    select_index = -1;
    select_name = wxEmptyString;
  }
  bypass = false;
  return true;
}
//-----------------------------------------------------------------------------
bool jfFilterPickerBase::SetSelected() {
  long itemindex = -1;
  itemindex = GetNextItem(itemindex,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
  // found
  if (itemindex<0) return false;
  select_name = GetItemText(itemindex);
  old_selection = select_index;
  select_index = itemindex;
  sloaded = true;
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfFilterPickerBase,wxListCtrl)
  EVT_LIST_ITEM_SELECTED(wxID_ANY,jfFilterPickerBase::OnItemSelect)
END_EVENT_TABLE()
//============================================================================
jfFilterPicker1::jfFilterPicker1(wxWindow* parent, wxStaticText* displaylink,
      wxWindowID id, const wxString& name):jfFilterPickerBase(parent,true,id,name) {
  // most of what we need to do has been handles by the base class
  assert(displaylink!=NULL);
  display = displaylink;
}
//---------------------------------------------------------------------------
// preoduces the selected filter name. properly escaped and wrapped in @
wxString jfFilterPicker1::InsertString() const {
  wxString outbuf;
  // starting
  if (!sloaded) return wxT("");
  // if we have a prefix, we encode that first
  if (!(prefix.IsEmpty())) {
    outbuf = EscapeString(prefix,EXPR_DELIMS);
    outbuf += wxT(';');
  }
  // next, we add the selected name
  outbuf += EscapeString(select_name,EXPR_DELIMS);
  // we wrap the results in the delimiters
  outbuf = wxT('@')+outbuf+wxT('@');
  // done
  return outbuf;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// the only used event
void jfFilterPicker1::OnItemSelect(wxListEvent& event) {
  wxString lbuffer;
  bool atest;
  // setting the label
  atest = SetSelected();
  assert(atest);
  lbuffer = InsertString();
  display->SetLabel(lbuffer);
}
//-------------------------------------------------------------------------
void jfFilterPicker1::DistribColumns(wxSizeEvent &event) {
  SetColWidth(4,3,6);
  event.Skip();
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// event table
BEGIN_EVENT_TABLE(jfFilterPicker1,jfFilterPickerBase)
  EVT_LIST_ITEM_SELECTED(wxID_ANY,jfFilterPicker1::OnItemSelect)
  EVT_SIZE(jfFilterPicker1::DistribColumns)
END_EVENT_TABLE()

//=============================================================================
jfGlobalFilterPicker::jfGlobalFilterPicker(wxWindow* parent,
    wxStaticText* displaylink, wxWindowID id, const wxString& name):
    wxPanel(parent,id,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,name) {
  // local variables
  assert(displaylink!=NULL);
  // creating the elements
  topsizer = new wxBoxSizer(wxHORIZONTAL);
  lbox = new wxListBox(this,wxID_ANY);
  fpicker = new jfFilterPicker1(this,displaylink,true);
  // arranging them
  topsizer->Add(lbox,2,wxALL|wxEXPAND,3);
  topsizer->Add(fpicker,5,wxTOP|wxBOTTOM|wxLEFT | wxEXPAND,3);
  SetSizer(topsizer);
  // initializing local variables
  loaded = false;
  proxy_link = NULL;
  sel_map = NULL;
  sel_mapname = wxT("");
}
//-----------------------------------------------------------------------------
/* we load the data in the Map of Maps we are given */
bool jfGlobalFilterPicker::LoadData(jfMapOfFiltermaps* inlink) {
  wxArrayString* names_of_maps;
  // local variables
  size_t mom_size;
  // first things first, we check what we get
  if (inlink==NULL) return false;
  // an empty map of maps is treated the same as nothing
  mom_size = inlink->size();
  if (mom_size==0) return false;
  // otherwise, we start
  proxy_link = inlink;
  names_of_maps = proxy_link->GetMapNames();
  assert(names_of_maps!=NULL);
  // loading the names of maps...
  lbox->Set(*names_of_maps);
  delete names_of_maps;
  // now we select the first one
  lbox->SetSelection(0);
  // and load
  LoadPicker(0);
  return true;
}
//----------------------------------------------------------------------------
// sets the data in the picker control to the map at the index
bool jfGlobalFilterPicker::LoadPicker(const size_t& pindex) {
  //local variables
  jfMapOfFiltermaps::iterator indexer;
  size_t counter, maxsize;
  // some initial checking
  assert(proxy_link!=NULL);
  maxsize = proxy_link->size();
  assert(maxsize>0);
  assert(pindex<maxsize);
  /* we loop thru the map, because the map key is string names,
  not integer indexes */
  indexer = proxy_link->begin();
  counter = 0;
  // the loop
  while (counter<pindex) {
    ++indexer;
    counter++;
  }
  // we are at the item, we set values...
  sel_mapname = indexer->first;
  sel_map = indexer->second;
  // the nest step is to load the data into the picker control
  fpicker->SetContents(sel_map,sel_mapname);
  // done (I think)
  return true;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool jfGlobalFilterPicker::IsFilterChosen() const {
  if (loaded) return fpicker->IsFilterSelected();
  else return false;
}
//----------------------------------------------------------------------------
wxString jfGlobalFilterPicker::CurrentMapname() const {
  if (loaded) return sel_mapname;
  else return wxT("");
}
//----------------------------------------------------------------------------
wxString jfGlobalFilterPicker::InsertString() const {
  if (loaded) return fpicker->InsertString();
  else return wxT("");
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGlobalFilterPicker::MapSelected(wxCommandEvent& event) {
  int sel_item;
  sel_item = lbox->GetSelection();
  LoadPicker(sel_item);
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfGlobalFilterPicker,wxPanel)
  EVT_LISTBOX(wxID_ANY,jfGlobalFilterPicker::MapSelected)
END_EVENT_TABLE()
//============================================================================
jfFilterPickerDialog::jfFilterPickerDialog(wxWindow* parent, wxWindowID id,
      const wxString& title ,const jfFilterMap* inlocal_map, wxString* inresult_ptr)
      :wxDialog(parent,id,title) {
  // first, we check on the inputs
  assert(inlocal_map!=NULL);
  assert(inresult_ptr);
  // assign the inputs
  local_map = inlocal_map;
  result_ptr = inresult_ptr;
  // we start  work on control creation
  pick_btn = new wxButton(this,wxID_OK,wxT("Pick"));
  cancel_btn = new wxButton(this,wxID_CANCEL,wxT("Cancel"));
  labelresult = new wxStaticText(this,-1,wxT("Filter Reference:"));
  showresult = new wxStaticText(this,-1,wxT(""));
  // create and arrange the bottom row
  bottom_sizer = new wxBoxSizer(wxHORIZONTAL);
  bottom_sizer->Add(pick_btn,0,wxALIGN_LEFT|wxALL,3);
  bottom_sizer->Add(cancel_btn,0,wxALIGN_LEFT|wxTOP|wxBOTTOM|wxLEFT,3);
  bottom_sizer->AddSpacer(8);
  bottom_sizer->Add(labelresult,0,wxALIGN_LEFT|wxALL,3);
  bottom_sizer->Add(showresult,1,wxALIGN_LEFT|wxTOP|wxBOTTOM|wxLEFT,3);
  // we create the notebook
  display_book = new wxNotebook(this,-1);
  // arranging things
  topsizer = new wxBoxSizer(wxVERTICAL);
  topsizer->Add(display_book,1,wxEXPAND|wxALL,3);
  topsizer->Add(bottom_sizer,0,wxALIGN_BOTTOM|wxALL,3);
  // we make the pages the pages
  local_page = new jfFilterPicker1(display_book,showresult,true);
  local_page->SetContents(local_map);
  global_page = new jfGlobalFilterPicker(display_book,showresult);
  global_page->LoadData(filterglobals::global_filters);
  // adding the pages to the notebook
  display_book->AddPage(local_page,wxT("Local Filters"));
  display_book->AddPage(global_page,wxT("Global Filters"));
  // now, set the sizer and recalculate
  SetSizer(topsizer);
  SetSize(700,350);
  Update();
  Centre();
  // done!
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfFilterPickerDialog::ProcessPick(wxCommandEvent& event) {
  (*result_ptr) = showresult->GetLabel();
  event.Skip();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfFilterPickerDialog,wxDialog)
  EVT_BUTTON(wxID_OK,jfFilterPickerDialog::ProcessPick)
END_EVENT_TABLE()
//============================================================================
jfGlobalFilterPickerDialog::jfGlobalFilterPickerDialog(wxWindow* parent, wxWindowID id,
      const wxString& title ,wxString* inresult_ptr):wxDialog(parent,id,title) {
  // first, we check on the inputs
  assert(inresult_ptr);
  // assign the inputs
  result_ptr = inresult_ptr;
  // we start  work on control creation
  pick_btn = new wxButton(this,wxID_OK,wxT("Pick"));
  cancel_btn = new wxButton(this,wxID_CANCEL,wxT("Cancel"));
  labelresult = new wxStaticText(this,-1,wxT("Filter Reference:"));
  showresult = new wxStaticText(this,-1,wxT(""));
  // create and arrange the bottom row
  bottom_sizer = new wxBoxSizer(wxHORIZONTAL);
  bottom_sizer->Add(pick_btn,0,wxALIGN_LEFT|wxALL,3);
  bottom_sizer->Add(cancel_btn,0,wxALIGN_LEFT|wxTOP|wxBOTTOM|wxLEFT,3);
  bottom_sizer->AddSpacer(8);
  bottom_sizer->Add(labelresult,0,wxALIGN_LEFT|wxALL,3);
  bottom_sizer->Add(showresult,1,wxALIGN_LEFT|wxTOP|wxBOTTOM|wxLEFT,3);
  // making the panel
  global_page = new jfGlobalFilterPicker(this,showresult);
  // arranging things
  topsizer = new wxBoxSizer(wxVERTICAL);
  topsizer->Add(global_page,1,wxEXPAND|wxALL,3);
  topsizer->Add(bottom_sizer,0,wxALIGN_BOTTOM|wxALL,3);
  // loading the data
  global_page->LoadData(filterglobals::global_filters);
  // now, set the sizer and recalculate
  SetSizer(topsizer);
  SetSize(550,350);
  Update();
  // done!
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void jfGlobalFilterPickerDialog::ProcessPick(wxCommandEvent& event) {
  (*result_ptr) = showresult->GetLabel();
  event.Skip();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfGlobalFilterPickerDialog,wxDialog)
  EVT_BUTTON(wxID_OK,jfGlobalFilterPickerDialog::ProcessPick)
END_EVENT_TABLE()
//*****************************************************************************
