/******************************************************************************
Name    :   progresspanel.cpp
Author  :   John Q Metro
Purpose :   The new progress panel, for displaying progress and button that control it.
Created :   November 12, 2013
Updated :   January 26, 2014
******************************************************************************/
#ifndef PROGRESSPANEL_H_INCLUDED
  #include "progresspanel.h"
#endif // PROGRESSPANEL_H_INCLUDED
//----------------------------------
#ifndef INTERFACE_IDS_H_INCLUDED
  #include "../interface_ids.h"
#endif // INTERFACE_IDS_H_INCLUDED
#ifndef LOGGING_H_INCLUDED
  #include "../../core/utils/logging.h"
#endif // LOGGING_H_INCLUDED
/*****************************************************************************/
bool IsStatusActive(const jf_PPS& instat, bool istop) {
  if (instat==jfp_NOTSTARTED) return false;
  if (instat==jfp_HALTED) return false;
  if (instat==jfp_DONE) return false;
  if (instat==jfp_FAILED) return false;
  if (istop && (instat==jfp_STOPPING)) return false;
  return true;
}
//========================================================================================
jfProgressPanelBase::jfProgressPanelBase(wxWindow* parent, wxString motif_name):wxPanel(
            parent,jfID_PROGPANEL,wxDefaultPosition,wxDefaultSize,wxTAB_TRAVERSAL,motif_name) {
  // setting up default values
  working = false;
  pageindex = 0;
  status = jfp_NOTSTARTED;
  thepauser = NULL;
  rpause = false;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// getting data
jfNewPauser* jfProgressPanelBase::GetPauser() {
  return thepauser;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// some special external methods
//----------------------------------
/* the start button does not lauch the actual download, but a message is
send up to a highter level. If the download is not launched, this method is called */
bool jfProgressPanelBase::CancelStart() {
  if (status!=jfp_STARTING) return false;
  SetStatus(jfp_NOTSTARTED);
  startstop_btn->Enable(true);
  return true;
}
//----------------------------------
// emergency halt order
bool jfProgressPanelBase::OrderStop() {
  const wxString fname = wxT("jfProgressPanelBase::OrderStop");
  // already stopped or stopping
  /**/jfXLogMessage(fname,1);
  if (!working) return false;
  /**/jfXLogMessage(fname,2);
  if (status==jfp_STOPPING) return false;
  /**/jfXLogMessage(fname,3);
  // disabling some buttons
  if (status!=jfp_PAUSING) pauseresume_btn->Enable(false);
  if (status==jfp_PAUSED) pauseresume_btn->SetLabel(wxT("Pause"));
  SetStatus(jfp_STOPPING);
  /**/jfXLogMessage(fname,4);
  startstop_btn->Enable(false);
  // doing the stop
  /**/jfXLogMessage(fname,5);
  thepauser->Stop();
  /**/jfXLogMessage(fname,6);
  PostStopping();
  return true;
}
//----------------------------------
bool jfProgressPanelBase::InProgress() const {
  return working;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for events generated from this panel
//----------------------------------
void jfProgressPanelBase::StartStopBtnHandler(wxCommandEvent& event) {
  // first, start or stop?
  const wxString fname = wxT("jfProgressPanelBase::StartStopBtnHandler");
  // start
  if (!working) {
    /**/jfXLogMessage(fname,1);
    if (status!=jfp_NOTSTARTED) Reset();
    thepauser = new jfNewPauser(this);
    startstop_btn->Enable(false);
    SetStatus(jfp_STARTING);
    // passing the start back up
    event.Skip();
  }
  // stop
  else {
    /**/jfXLogMessage(fname,2);
    OrderStop();
  }
}
//----------------------------------
void jfProgressPanelBase::PauseResumeBtnHandler(wxCommandEvent& event) {
  const wxString fname = wxT("jfProgressPanelBase::PauseResumeBtnHandler");
  // only pause when working, only resume when paused
  assert((status==jfp_WORKING) || (status==jfp_PAUSED));
  /**/jfXLogMessage(fname,1);
  // pause
  if (status==jfp_WORKING) {
    pauseresume_btn->Enable(false);
    SetStatus(jfp_PAUSING);
    thepauser->Pause();
    rpause = false;
  }
  // resume
  else {
    pauseresume_btn->SetLabel(wxT("Pause"));
    pauseresume_btn->Enable(false);
    thepauser->Resume();
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for events that come from the getter
//----------------------------------
void jfProgressPanelBase::HandleRetry(wxCommandEvent& event) {
  //pretty simple
  stat_label->SetLabel(wxT("RETRYING"));
}
//----------------------------------
// a pause that come from the thread, rather than the button
void jfProgressPanelBase::HandleRequestPause(wxCommandEvent& event) {
  if (status==jfp_WORKING) {
    pauseresume_btn->Enable(false);
    SetStatus(jfp_PAUSING);
    thepauser->Pause();
    rpause = true;
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for events that come from the pauser
//----------------------------------
void jfProgressPanelBase::HandlePause(wxCommandEvent& event) {
  assert(status==jfp_PAUSING);
  pauseresume_btn->SetLabel(wxT("Resume"));
  SetStatus(jfp_PAUSED);
  if (rpause) stat_label->SetLabel(wxT("AUTO PAUSED"));
  pauseresume_btn->Enable();
  rpause = false;
}
//----------------------------------
void jfProgressPanelBase::HandleResume(wxCommandEvent& event) {
  SetStatus(jfp_WORKING);
  pauseresume_btn->Enable(true);
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// handlers for thread events
//----------------------------------
// confirmation the thread is working
void jfProgressPanelBase::HandleStart(jfThreadEvent& tevent) {
  SetStatus(jfp_WORKING);
  working = true;
  startstop_btn->SetLabel(wxT("Stop"));
  pauseresume_btn->Enable(true);
  startstop_btn->Enable(true);
  PostStarted();
}
//----------------------------------
// a successful end
void jfProgressPanelBase::HandleEnd(jfThreadEvent& tevent) {
  StopAction(jfp_DONE);
  PostFinished();
}
//----------------------------------
void jfProgressPanelBase::HandleTotalCount(jfThreadEvent& tevent) {
  page_display->SetCount(tevent.GetInt1());
}
//----------------------------------
void jfProgressPanelBase::HandleLabels(jfThreadEvent& tevent) {
  page_display->ChangeTypestring(tevent.GetString1(),tevent.GetString2());
}
//----------------------------------
void jfProgressPanelBase::HandleNewItem(jfThreadEvent& tevent) {
  pageindex++;
  page_display->SetNew(tevent.GetInt1(),tevent.GetInt2(),tevent.GetString1());
}
//----------------------------------
void jfProgressPanelBase::HandleAction(jfThreadEvent& tevent) {
  page_display->ChangeAction(tevent.GetString1());
}
//----------------------------------
void jfProgressPanelBase::HandleHalt(jfThreadEvent& tevent) {
  StopAction(jfp_HALTED);
  PostFaliure(true);
}
//----------------------------------
void jfProgressPanelBase::HandleReset(jfThreadEvent& tevent) {
  page_display->SetToZero();
  pageindex = 0;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// internal helper methods
//----------------------------------
bool jfProgressPanelBase::SetupWidgets() {
  // setting up widgets
  page_display = new jfLabeledGauge(this,20);
  startstop_btn = new wxButton(this,jfID_STARTSTOPBTN,wxT("Start"));
  pauseresume_btn = new wxButton(this,jfID_PAUSERESUME,wxT("Pause"));
  pauseresume_btn->Enable(false);
  stat_label = new wxStaticText(this,-1,wxT("NOT STARTED"));
  // button height
  wxSize ssize = startstop_btn->GetMinSize();
  dbheight = ssize.GetHeight();
  return true;
}
//----------------------------------
bool jfProgressPanelBase::ChangeButtonHights(size_t nheight, wxButton* extrabtn) {
  if (nheight<5) return false;
  // changing the button height
  wxSize ssize = startstop_btn->GetMinSize();
  ssize.SetHeight(nheight);
  startstop_btn->SetMinSize(ssize);
  pauseresume_btn->SetMinSize(ssize);
  // extra
  if (extrabtn!=NULL) {
    ssize = extrabtn->GetMinSize();
    ssize.SetHeight(nheight);
    extrabtn->SetMinSize(ssize);
  }
  return true;
}
//----------------------------------
bool jfProgressPanelBase::BaseReset() {
  if (working) return false;
  SetStatus(jfp_NOTSTARTED);
  page_display->SetToZero();
  pageindex = 0;
  rpause = false;
  return true;
}
//----------------------------------
// thing to do when we stop
bool jfProgressPanelBase::StopAction(jf_PPS instatus) {
  if (!working) return false;
  pauseresume_btn->Enable(false);
  startstop_btn->SetLabel(wxT("Start"));
  startstop_btn->Enable(true);
  working = false;
  SetStatus(instatus);
  return true;
}
//----------------------------------
void jfProgressPanelBase::SetStatus(jf_PPS instatus) {
  if (instatus==jfp_NOTSTARTED) stat_label->SetLabel(wxT("NOT STARTED"));
  else if (instatus==jfp_STARTING) stat_label->SetLabel(wxT("STARTING"));
  else if (instatus==jfp_WORKING) stat_label->SetLabel(wxT("WORKING"));
  else if (instatus==jfp_PAUSING) stat_label->SetLabel(wxT("PAUSING"));
  else if (instatus==jfp_PAUSED) stat_label->SetLabel(wxT("PAUSED"));
  else if (instatus==jfp_STOPPING) stat_label->SetLabel(wxT("STOPPING"));
  else if (instatus==jfp_HALTED) stat_label->SetLabel(wxT("HALTED"));
  else if (instatus==jfp_FAILED) stat_label->SetLabel(wxT("FAILED"));
  else if (instatus==jfp_DONE) stat_label->SetLabel(wxT("DONE"));
  else assert(false);
  status = instatus;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfProgressPanelBase,wxPanel)
  EVT_BUTTON (jfID_STARTSTOPBTN, jfProgressPanelBase::StartStopBtnHandler  )
  EVT_BUTTON (jfID_PAUSERESUME , jfProgressPanelBase::PauseResumeBtnHandler)
  EVT_COMMAND (-1,jf_DOWNL_RETRY_EVENT ,jfProgressPanelBase::HandleRetry       )
  EVT_COMMAND (-1,jf_DOWNL_RPAUSE_EVENT,jfProgressPanelBase::HandleRequestPause)
  EVT_COMMAND (-1,jf_DOWNL_PAUSE_EVENT ,jfProgressPanelBase::HandlePause )
  EVT_COMMAND (-1,jf_DOWNL_RESUME_EVENT,jfProgressPanelBase::HandleResume)
  EVT_JF_THREAD (-1,jf_DOWNL_START_EVENT ,jfProgressPanelBase::HandleStart     )
  EVT_JF_THREAD (-1,jf_DOWNL_END_EVENT   ,jfProgressPanelBase::HandleEnd       )
  EVT_JF_THREAD (-1,jf_DOWNL_PCOUNT_EVENT,jfProgressPanelBase::HandleTotalCount)
  EVT_JF_THREAD (-1,jf_DOWNL_LABELS_EVENT,jfProgressPanelBase::HandleLabels    )
  EVT_JF_THREAD (-1,jf_DOWNL_PSTART_EVENT,jfProgressPanelBase::HandleNewItem   )
  EVT_JF_THREAD (-1,jf_DOWNL_PPROC_EVENT ,jfProgressPanelBase::HandleAction    )
  EVT_JF_THREAD (-1,jf_DOWNL_PHALT_EVENT ,jfProgressPanelBase::HandleHalt      )
  EVT_JF_THREAD (-1,jf_DOWNL_RESET_EVENT ,jfProgressPanelBase::HandleReset     )
END_EVENT_TABLE()
/*****************************************************************************/
jfProgressPanelSingle::jfProgressPanelSingle(wxWindow* parent, wxString motif_name):
        jfProgressPanelBase(parent,motif_name) {
  // setup
  topsizer = new wxStaticBoxSizer(wxHORIZONTAL,this,wxT("Status Information"));
  SetupWidgets();
  SetupSizers();
  Layout();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++
// reset method
bool jfProgressPanelSingle::Reset() {
  return BaseReset();
}
//+++++++++++++++++++++++++++++++++++++++++++++++++
// implemented handler
void jfProgressPanelSingle::HandleDone(jfThreadEvent& tevent) {
  // variables
  jfDown::ResultAction res_action;
  wxString fres;
  wxString errstr;
  // starting
  res_action = tevent.GetResultAction();
  if ((res_action==jfDown::FAIL)||(res_action==jfDown::SKIP)) {
    fres = tevent.GetString1();
    errstr = tevent.GetString2();
  }
  // two main options here, we go on, or we stop
  if (res_action==jfDown::PROCEED) {
    SetStatus(status);
    page_display->SetDone(wxT("Done with"));
  }
  else if (res_action==jfDown::SKIP) {
    SetStatus(status);
    page_display->SetDone(wxT("Skipping"));
  }
  else {
    // specials
    if (res_action==jfDown::HALT) {
      StopAction(jfp_HALTED);
      PostFaliure(true);
    }
    else {
      StopAction(jfp_FAILED);
      page_display->ChangeLabel(fres+ wxT(" :") + errstr);
      PostFaliure(false);
    }
  }
}
//+++++++++++++++++++++++++++++++++++++++++++++++++
// resizer methods
//------------------------------
bool jfProgressPanelSingle::ReduceSize() {
  // varibales
  wxSize ssize;
  const size_t bsize = 10;
  const size_t fpsize = 8;
  const size_t bheight = 24;
  // changing the button height
  ChangeButtonHights(bheight);
  page_display->ChangeSizes(bsize,fpsize);
  return true;
}
//------------------------------
bool jfProgressPanelSingle::UnReduceSize() {
  ChangeButtonHights(dbheight);
  page_display->RevertSizes();
  return true;
}
//+++++++++++++++++++++++++++++++++++++++++++++++++
// internal implemented helper methods
//------------------------------
void jfProgressPanelSingle::SetupSizers() {
  // here, we arrange the gui elemnts within the sizer
  topsizer->Add(stat_label,0,wxALIGN_CENTRE_VERTICAL|wxALL,3);
  topsizer->Add(page_display,1,wxTOP|wxBOTTOM,3);
  topsizer->AddSpacer(6);
  topsizer->Add(startstop_btn,0,wxALL,3);
  topsizer->Add(pauseresume_btn,0,wxALL,3);
  // finishing
  SetSizer(topsizer);
}
//------------------------------
void jfProgressPanelSingle::PostFinished() {
  wxCommandEvent outevent(jf_PP_SRCHDONE_EVENT,GetId());
  wxPostEvent(GetParent(),outevent);
}
//------------------------------
void jfProgressPanelSingle::PostFaliure(bool halted) {
  wxCommandEvent outevent(jf_PP_SRCHFAIL_EVENT,GetId());
  outevent.SetInt(halted);
  wxPostEvent(GetParent(),outevent);
}
//------------------------------
void jfProgressPanelSingle::PostStopping() {}
//------------------------------
void jfProgressPanelSingle::PostStarted() {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
BEGIN_EVENT_TABLE(jfProgressPanelSingle,jfProgressPanelBase)
  EVT_JF_THREAD (-1,jf_DOWNL_PEND_EVENT,jfProgressPanelBase::HandleDone )
END_EVENT_TABLE()

/*****************************************************************************/
